@@ -4,6 +4,7 @@
 #include <QWidget>
 #include <QPointer>
 #include <QDoubleSpinBox>
+#include <QStackedWidget>
 #include "balance-slider.hpp"
 
 class QGridLayout;
@@ -12,17 +13,27 @@ class QSpinBox;
 class QCheckBox;
 class QComboBox;
 
+enum class VolumeType {
+	dB,
+	Percent,
+};
+
 class OBSAdvAudioCtrl : public QObject {
 	Q_OBJECT
 
 private:
 	OBSSource source;
 
+	QPointer<QWidget> activeContainer;
 	QPointer<QWidget> forceMonoContainer;
 	QPointer<QWidget> mixerContainer;
 	QPointer<QWidget> balanceContainer;
 
+	QPointer<QLabel> iconLabel;
 	QPointer<QLabel> nameLabel;
+	QPointer<QLabel> active;
+	QPointer<QStackedWidget> stackedWidget;
+	QPointer<QSpinBox> percent;
 	QPointer<QDoubleSpinBox> volume;
 	QPointer<QCheckBox> forceMono;
 	QPointer<BalanceSlider> balance;
@@ -40,12 +51,21 @@ private:
 	OBSSignal volChangedSignal;
 	OBSSignal syncOffsetSignal;
 	OBSSignal flagsSignal;
+	OBSSignal monitoringTypeSignal;
 	OBSSignal mixersSignal;
+	OBSSignal activateSignal;
+	OBSSignal deactivateSignal;
+	OBSSignal balChangedSignal;
 
+	static void OBSSourceActivated(void *param, calldata_t *calldata);
+	static void OBSSourceDeactivated(void *param, calldata_t *calldata);
 	static void OBSSourceFlagsChanged(void *param, calldata_t *calldata);
 	static void OBSSourceVolumeChanged(void *param, calldata_t *calldata);
 	static void OBSSourceSyncChanged(void *param, calldata_t *calldata);
+	static void OBSSourceMonitoringTypeChanged(void *param,
+						   calldata_t *calldata);
 	static void OBSSourceMixersChanged(void *param, calldata_t *calldata);
+	static void OBSSourceBalanceChanged(void *param, calldata_t *calldata);
 
 public:
 	OBSAdvAudioCtrl(QGridLayout *layout, obs_source_t *source_);
@@ -54,13 +74,20 @@ public:
 	inline obs_source_t *GetSource() const { return source; }
 	void ShowAudioControl(QGridLayout *layout);
 
+	void SetVolumeWidget(VolumeType type);
+	void SetIconVisible(bool visible);
+
 public slots:
+	void SourceActiveChanged(bool active);
 	void SourceFlagsChanged(uint32_t flags);
 	void SourceVolumeChanged(float volume);
 	void SourceSyncChanged(int64_t offset);
+	void SourceMonitoringTypeChanged(int type);
 	void SourceMixersChanged(uint32_t mixers);
+	void SourceBalanceChanged(int balance);
 
 	void volumeChanged(double db);
+	void percentChanged(int percent);
 	void downmixMonoChanged(bool checked);
 	void balanceChanged(int val);
 	void syncOffsetChanged(int milliseconds);
