@@ -22,10 +22,10 @@ protected:
 		std::unique_ptr<obs_properties_t, properties_delete_t>;
 
 	properties_t props;
-	OBSData oldData;
+	OBSDataAutoRelease oldData;
 
 	void SaveOldProperties(obs_source_t *source);
-	void SetUndoProperties(obs_source_t *source);
+	void SetUndoProperties(obs_source_t *source, bool repeatable = false);
 
 public:
 	SourceToolbar(QWidget *parent, OBSSource source);
