@@ -48,7 +48,9 @@
 #include <thread>
 #endif
 
-using namespace std;
+using std::string;
+using std::vector;
+using std::ostringstream;
 
 #ifdef __linux__
 void RunningInstanceCheck(bool &already_running)
@@ -65,12 +67,9 @@ void RunningInstanceCheck(bool &already_running)
 	struct sockaddr_un bindInfo;
 	memset(&bindInfo, 0, sizeof(sockaddr_un));
 	bindInfo.sun_family = AF_LOCAL;
-	char *abstactSockName = NULL;
-	asprintf(&abstactSockName, "%s %d %s", "/com/obsproject", getpid(),
+	snprintf(bindInfo.sun_path + 1, sizeof(bindInfo.sun_path) - 1,
+		 "%s %d %s", "/com/obsproject", getpid(),
 		 App()->GetVersionString().c_str());
-	memmove(bindInfo.sun_path + 1, abstactSockName,
-		strlen(abstactSockName));
-	free(abstactSockName);
 
 	int bindErr = bind(uniq, (struct sockaddr *)&bindInfo,
 			   sizeof(struct sockaddr_un));
@@ -137,7 +136,7 @@ const char *RunOnce::thr_name = "OBS runonce";
 void PIDFileCheck(bool &already_running)
 {
 	std::string tmpfile_name =
-		"/tmp/obs-studio.lock." + to_string(geteuid());
+		"/tmp/obs-studio.lock." + std::to_string(geteuid());
 	int fd = open(tmpfile_name.c_str(), O_RDWR | O_CREAT | O_EXLOCK, 0600);
 	if (fd == -1) {
 		already_running = true;
@@ -214,7 +213,6 @@ string GetDefaultVideoSavePath()
 
 vector<string> GetPreferredLocales()
 {
-	setlocale(LC_ALL, "");
 	vector<string> matched;
 	string messages = setlocale(LC_MESSAGES, NULL);
 	if (!messages.size() || messages == "C" || messages == "POSIX")
@@ -252,3 +250,9 @@ void SetAlwaysOnTop(QWidget *window, bool enable)
 	window->setWindowFlags(flags);
 	window->show();
 }
+
+bool SetDisplayAffinitySupported(void)
+{
+	// Not implemented yet
+	return false;
+}
