@@ -62,7 +62,6 @@ OBSBasicFilters::OBSBasicFilters(QWidget *parent, OBSSource source_)
 	setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
 	ui->setupUi(this);
-	UpdateFilters();
 
 	ui->asyncFilters->setItemDelegate(
 		new VisibilityItemDelegate(ui->asyncFilters));
@@ -119,11 +118,25 @@ OBSBasicFilters::OBSBasicFilters(QWidget *parent, OBSSource source_)
 	if (audioOnly) {
 		ui->effectWidget->setVisible(false);
 		ui->separatorLine->setVisible(false);
+		UpdateSplitter(false);
+	}
+
+	if (async && !audioOnly && ui->asyncFilters->count() == 0 &&
+	    ui->effectFilters->count() != 0) {
+		ui->effectFilters->setFocus();
 	}
 
 	if (audioOnly || (audio && !async))
 		ui->asyncLabel->setText(QTStr("Basic.Filters.AudioFilters"));
 
+	if (async && audio && ui->asyncFilters->count() == 0) {
+		UpdateSplitter(false);
+	} else if (!audioOnly) {
+		UpdateSplitter();
+	}
+
+	obs_source_inc_showing(source);
+
 	auto addDrawCallback = [this]() {
 		obs_display_add_draw_callback(ui->preview->GetDisplay(),
 					      OBSBasicFilters::DrawPreview,
@@ -142,7 +155,6 @@ OBSBasicFilters::OBSBasicFilters(QWidget *parent, OBSSource source_)
 				addDrawCallback);
 	} else {
 		ui->rightLayout->setContentsMargins(0, noPreviewMargin, 0, 0);
-		ui->rightContainerLayout->insertStretch(1);
 		ui->preview->hide();
 	}
 
@@ -165,10 +177,13 @@ OBSBasicFilters::OBSBasicFilters(QWidget *parent, OBSSource source_)
 	renameAsync->setShortcut({Qt::Key_F2});
 	renameEffect->setShortcut({Qt::Key_F2});
 #endif
+
+	UpdateFilters();
 }
 
 OBSBasicFilters::~OBSBasicFilters()
 {
+	obs_source_dec_showing(source);
 	ClearListItems(ui->asyncFilters);
 	ClearListItems(ui->effectFilters);
 }
@@ -192,84 +207,81 @@ inline OBSSource OBSBasicFilters::GetFilter(int row, bool async)
 	return v.value<OBSSource>();
 }
 
+void FilterChangeUndoRedo(void *vp, obs_data_t *nd_old_settings,
+			  obs_data_t *new_settings)
+{
+	obs_source_t *source = reinterpret_cast<obs_source_t *>(vp);
+	obs_source_t *parent = obs_filter_get_parent(source);
+	const char *source_name = obs_source_get_name(source);
+	OBSBasic *main = OBSBasic::Get();
+
+	OBSDataAutoRelease redo_wrapper = obs_data_create();
+	obs_data_set_string(redo_wrapper, "name", source_name);
+	obs_data_set_string(redo_wrapper, "settings",
+			    obs_data_get_json(new_settings));
+	obs_data_set_string(redo_wrapper, "parent",
+			    obs_source_get_name(parent));
+
+	OBSDataAutoRelease filter_settings = obs_source_get_settings(source);
+
+	OBSDataAutoRelease undo_wrapper = obs_data_create();
+	obs_data_set_string(undo_wrapper, "name", source_name);
+	obs_data_set_string(undo_wrapper, "settings",
+			    obs_data_get_json(nd_old_settings));
+	obs_data_set_string(undo_wrapper, "parent",
+			    obs_source_get_name(parent));
+
+	auto undo_redo = [](const std::string &data) {
+		OBSDataAutoRelease dat =
+			obs_data_create_from_json(data.c_str());
+		OBSSourceAutoRelease parent_source = obs_get_source_by_name(
+			obs_data_get_string(dat, "parent"));
+		const char *filter_name = obs_data_get_string(dat, "name");
+		OBSSourceAutoRelease filter = obs_source_get_filter_by_name(
+			parent_source, filter_name);
+		OBSDataAutoRelease new_settings = obs_data_create_from_json(
+			obs_data_get_string(dat, "settings"));
+
+		OBSDataAutoRelease current_settings =
+			obs_source_get_settings(filter);
+		obs_data_clear(current_settings);
+
+		obs_source_update(filter, new_settings);
+		obs_source_update_properties(filter);
+	};
+
+	main->undo_s.enable();
+
+	std::string name = std::string(obs_source_get_name(source));
+	std::string undo_data = obs_data_get_json(undo_wrapper);
+	std::string redo_data = obs_data_get_json(redo_wrapper);
+	main->undo_s.add_action(QTStr("Undo.Filters").arg(name.c_str()),
+				undo_redo, undo_redo, undo_data, redo_data);
+
+	obs_source_update(source, new_settings);
+}
+
 void OBSBasicFilters::UpdatePropertiesView(int row, bool async)
 {
+	OBSSource filter = GetFilter(row, async);
+	if (filter && view && view->IsObject(filter)) {
+		/* do not recreate properties view if already using a view
+		 * with the same object */
+		return;
+	}
+
 	if (view) {
 		updatePropertiesSignal.Disconnect();
-		ui->rightLayout->removeWidget(view);
+		ui->propertiesFrame->setVisible(false);
+		view->hide();
 		view->deleteLater();
 		view = nullptr;
 	}
 
-	OBSSource filter = GetFilter(row, async);
 	if (!filter)
 		return;
 
-	obs_data_t *settings = obs_source_get_settings(filter);
-
-	auto filter_change = [](void *vp, obs_data_t *nd_old_settings,
-				obs_data_t *new_settings) {
-		obs_source_t *source = reinterpret_cast<obs_source_t *>(vp);
-		obs_source_t *parent = obs_filter_get_parent(source);
-		const char *source_name = obs_source_get_name(source);
-		OBSBasic *main = OBSBasic::Get();
-
-		obs_data_t *redo_wrapper = obs_data_create();
-		obs_data_set_string(redo_wrapper, "name", source_name);
-		obs_data_set_string(redo_wrapper, "settings",
-				    obs_data_get_json(new_settings));
-		obs_data_set_string(redo_wrapper, "parent",
-				    obs_source_get_name(parent));
-
-		obs_data_t *filter_settings = obs_source_get_settings(source);
-		obs_data_t *old_settings =
-			obs_data_get_defaults(filter_settings);
-		obs_data_apply(old_settings, nd_old_settings);
-
-		obs_data_t *undo_wrapper = obs_data_create();
-		obs_data_set_string(undo_wrapper, "name", source_name);
-		obs_data_set_string(undo_wrapper, "settings",
-				    obs_data_get_json(old_settings));
-		obs_data_set_string(undo_wrapper, "parent",
-				    obs_source_get_name(parent));
-
-		auto undo_redo = [](const std::string &data) {
-			obs_data_t *dat =
-				obs_data_create_from_json(data.c_str());
-			obs_source_t *parent_source = obs_get_source_by_name(
-				obs_data_get_string(dat, "parent"));
-			const char *filter_name =
-				obs_data_get_string(dat, "name");
-			obs_source_t *filter = obs_source_get_filter_by_name(
-				parent_source, filter_name);
-			obs_data_t *settings = obs_data_create_from_json(
-				obs_data_get_string(dat, "settings"));
-
-			obs_source_update(filter, settings);
-			obs_source_update_properties(filter);
-
-			obs_data_release(dat);
-			obs_data_release(settings);
-			obs_source_release(filter);
-			obs_source_release(parent_source);
-		};
-
-		main->undo_s.enable();
-
-		std::string name = std::string(obs_source_get_name(source));
-		std::string undo_data = obs_data_get_json(undo_wrapper);
-		std::string redo_data = obs_data_get_json(redo_wrapper);
-		main->undo_s.add_action(QTStr("Undo.Filters").arg(name.c_str()),
-					undo_redo, undo_redo, undo_data,
-					redo_data);
-
-		obs_data_release(redo_wrapper);
-		obs_data_release(undo_wrapper);
-		obs_data_release(old_settings);
-		obs_data_release(filter_settings);
-
-		obs_source_update(source, new_settings);
-	};
+	OBSDataAutoRelease settings = obs_source_get_settings(filter);
 
 	auto disabled_undo = [](void *vp, obs_data_t *settings) {
 		OBSBasic *main =
@@ -280,20 +292,18 @@ void OBSBasicFilters::UpdatePropertiesView(int row, bool async)
 	};
 
 	view = new OBSPropertiesView(
-		settings, filter,
+		settings.Get(), filter,
 		(PropertiesReloadCallback)obs_source_properties,
-		(PropertiesUpdateCallback)filter_change,
+		(PropertiesUpdateCallback)FilterChangeUndoRedo,
 		(PropertiesVisualUpdateCb)disabled_undo);
 
 	updatePropertiesSignal.Connect(obs_source_get_signal_handler(filter),
 				       "update_properties",
 				       OBSBasicFilters::UpdateProperties, this);
 
-	obs_data_release(settings);
-
-	view->setMaximumHeight(250);
 	view->setMinimumHeight(150);
-	ui->rightLayout->addWidget(view);
+	UpdateSplitter();
+	ui->propertiesLayout->addWidget(view);
 	view->show();
 }
 
@@ -445,6 +455,28 @@ void OBSBasicFilters::UpdateFilters()
 	main->SaveProject();
 }
 
+void OBSBasicFilters::UpdateSplitter()
+{
+	bool show_splitter_frame =
+		ui->asyncFilters->count() + ui->effectFilters->count() > 0;
+	UpdateSplitter(show_splitter_frame);
+}
+
+void OBSBasicFilters::UpdateSplitter(bool show_splitter_frame)
+{
+	bool show_splitter_handle = show_splitter_frame;
+	uint32_t caps = obs_source_get_output_flags(source);
+	if ((caps & OBS_SOURCE_VIDEO) == 0)
+		show_splitter_handle = false;
+
+	for (int i = 0; i < ui->rightLayout->count(); i++) {
+		QSplitterHandle *hndl = ui->rightLayout->handle(i);
+		hndl->setEnabled(show_splitter_handle);
+	}
+
+	ui->propertiesFrame->setVisible(show_splitter_frame);
+}
+
 static bool filter_compatible(bool async, uint32_t sourceFlags,
 			      uint32_t filterFlags)
 {
@@ -455,7 +487,8 @@ static bool filter_compatible(bool async, uint32_t sourceFlags,
 	bool audioOnly = (sourceFlags & OBS_SOURCE_VIDEO) == 0;
 	bool asyncSource = (sourceFlags & OBS_SOURCE_ASYNC) != 0;
 
-	if (async && ((audioOnly && filterVideo) || (!audio && !asyncSource)))
+	if (async && ((audioOnly && filterVideo) || (!audio && !asyncSource) ||
+		      (filterAudio && !audio)))
 		return false;
 
 	return (async && (filterAudio || filterAsync)) ||
@@ -529,7 +562,7 @@ QMenu *OBSBasicFilters::CreateAddFilterPopupMenu(bool async)
 void OBSBasicFilters::AddNewFilter(const char *id)
 {
 	if (id && *id) {
-		obs_source_t *existing_filter;
+		OBSSourceAutoRelease existing_filter;
 		string name = obs_source_get_display_name(id);
 
 		QString placeholder = QString::fromStdString(name);
@@ -537,7 +570,6 @@ void OBSBasicFilters::AddNewFilter(const char *id)
 		int i = 2;
 		while ((existing_filter = obs_source_get_filter_by_name(
 				source, QT_TO_UTF8(text)))) {
-			obs_source_release(existing_filter);
 			text = QString("%1 %2").arg(placeholder).arg(i++);
 		}
 
@@ -560,12 +592,11 @@ void OBSBasicFilters::AddNewFilter(const char *id)
 		if (existing_filter) {
 			OBSMessageBox::warning(this, QTStr("NameExists.Title"),
 					       QTStr("NameExists.Text"));
-			obs_source_release(existing_filter);
 			AddNewFilter(id);
 			return;
 		}
 
-		obs_data_t *wrapper = obs_data_create();
+		OBSDataAutoRelease wrapper = obs_data_create();
 		obs_data_set_string(wrapper, "sname",
 				    obs_source_get_name(source));
 		obs_data_set_string(wrapper, "fname", name.c_str());
@@ -592,31 +623,26 @@ void OBSBasicFilters::AddNewFilter(const char *id)
 			obs_source_release(filter);
 		};
 
-		obs_data_t *rwrapper = obs_data_create();
+		OBSDataAutoRelease rwrapper = obs_data_create();
 		obs_data_set_string(rwrapper, "sname",
 				    obs_source_get_name(source));
 		auto redo = [scene_name, id = std::string(id),
 			     name](const std::string &data) {
-			obs_source_t *ssource =
+			OBSSourceAutoRelease ssource =
 				obs_get_source_by_name(scene_name.c_str());
 			reinterpret_cast<OBSBasic *>(App()->GetMainWindow())
-				->SetCurrentScene(ssource, true);
-			obs_source_release(ssource);
+				->SetCurrentScene(ssource.Get(), true);
 
-			obs_data_t *dat =
+			OBSDataAutoRelease dat =
 				obs_data_create_from_json(data.c_str());
-			obs_source_t *source = obs_get_source_by_name(
+			OBSSourceAutoRelease source = obs_get_source_by_name(
 				obs_data_get_string(dat, "sname"));
 
-			obs_source_t *filter = obs_source_create(
+			OBSSourceAutoRelease filter = obs_source_create(
 				id.c_str(), name.c_str(), nullptr, nullptr);
 			if (filter) {
 				obs_source_filter_add(source, filter);
-				obs_source_release(filter);
 			}
-
-			obs_data_release(dat);
-			obs_source_release(source);
 		};
 
 		std::string undo_data(obs_data_get_json(wrapper));
@@ -624,10 +650,7 @@ void OBSBasicFilters::AddNewFilter(const char *id)
 		main->undo_s.add_action(QTStr("Undo.Add").arg(name.c_str()),
 					undo, redo, undo_data, redo_data);
 
-		obs_data_release(wrapper);
-		obs_data_release(rwrapper);
-
-		obs_source_t *filter =
+		OBSSourceAutoRelease filter =
 			obs_source_create(id, name.c_str(), nullptr, nullptr);
 		if (filter) {
 			const char *sourceName = obs_source_get_name(source);
@@ -638,7 +661,6 @@ void OBSBasicFilters::AddNewFilter(const char *id)
 			     name.c_str(), id, sourceName);
 
 			obs_source_filter_add(source, filter);
-			obs_source_release(filter);
 		}
 	}
 }
@@ -764,6 +786,7 @@ static bool QueryRemove(QWidget *parent, obs_source_t *source)
 
 void OBSBasicFilters::on_addAsyncFilter_clicked()
 {
+	ui->asyncFilters->setFocus();
 	QScopedPointer<QMenu> popup(CreateAddFilterPopupMenu(true));
 	if (popup)
 		popup->exec(QCursor::pos());
@@ -806,6 +829,7 @@ void OBSBasicFilters::on_asyncFilters_currentRowChanged(int row)
 
 void OBSBasicFilters::on_addEffectFilter_clicked()
 {
+	ui->effectFilters->setFocus();
 	QScopedPointer<QMenu> popup(CreateAddFilterPopupMenu(false));
 	if (popup)
 		popup->exec(QCursor::pos());
@@ -942,14 +966,13 @@ void OBSBasicFilters::DuplicateItem(QListWidgetItem *item)
 {
 	OBSSource filter = item->data(Qt::UserRole).value<OBSSource>();
 	string name = obs_source_get_name(filter);
-	obs_source_t *existing_filter;
+	OBSSourceAutoRelease existing_filter;
 
 	QString placeholder = QString::fromStdString(name);
 	QString text{placeholder};
 	int i = 2;
 	while ((existing_filter = obs_source_get_filter_by_name(
 			source, QT_TO_UTF8(text)))) {
-		obs_source_release(existing_filter);
 		text = QString("%1 %2").arg(placeholder).arg(i++);
 	}
 
@@ -970,12 +993,11 @@ void OBSBasicFilters::DuplicateItem(QListWidgetItem *item)
 	if (existing_filter) {
 		OBSMessageBox::warning(this, QTStr("NameExists.Title"),
 				       QTStr("NameExists.Text"));
-		obs_source_release(existing_filter);
 		DuplicateItem(item);
 		return;
 	}
 	bool enabled = obs_source_enabled(filter);
-	obs_source_t *new_filter =
+	OBSSourceAutoRelease new_filter =
 		obs_source_duplicate(filter, name.c_str(), false);
 	if (new_filter) {
 		const char *sourceName = obs_source_get_name(source);
@@ -986,7 +1008,6 @@ void OBSBasicFilters::DuplicateItem(QListWidgetItem *item)
 		     name.c_str(), id, name.c_str(), sourceName);
 		obs_source_set_enabled(new_filter, enabled);
 		obs_source_filter_add(source, new_filter);
-		obs_source_release(new_filter);
 	}
 }
 
@@ -1031,7 +1052,7 @@ void OBSBasicFilters::FilterNameEdited(QWidget *editor, QListWidget *list)
 
 	const char *prevName = obs_source_get_name(filter);
 	bool sameName = (name == prevName);
-	obs_source_t *foundFilter = nullptr;
+	OBSSourceAutoRelease foundFilter = nullptr;
 
 	if (!sameName)
 		foundFilter =
@@ -1044,8 +1065,6 @@ void OBSBasicFilters::FilterNameEdited(QWidget *editor, QListWidget *list)
 			OBSMessageBox::information(window(),
 						   QTStr("NameExists.Title"),
 						   QTStr("NameExists.Text"));
-			obs_source_release(foundFilter);
-
 		} else if (name.empty()) {
 			OBSMessageBox::information(window(),
 						   QTStr("NoNameEntered.Title"),
@@ -1066,36 +1085,32 @@ void OBSBasicFilters::FilterNameEdited(QWidget *editor, QListWidget *list)
 				->GetCurrentSceneSource());
 		auto undo = [scene_name, prev = std::string(prevName),
 			     name](const std::string &data) {
-			obs_source_t *ssource =
+			OBSSourceAutoRelease ssource =
 				obs_get_source_by_name(scene_name.c_str());
 			reinterpret_cast<OBSBasic *>(App()->GetMainWindow())
-				->SetCurrentScene(ssource, true);
-			obs_source_release(ssource);
+				->SetCurrentScene(ssource.Get(), true);
 
-			obs_source_t *source =
+			OBSSourceAutoRelease source =
 				obs_get_source_by_name(data.c_str());
-			obs_source_t *filter = obs_source_get_filter_by_name(
-				source, name.c_str());
+			OBSSourceAutoRelease filter =
+				obs_source_get_filter_by_name(source,
+							      name.c_str());
 			obs_source_set_name(filter, prev.c_str());
-			obs_source_release(source);
-			obs_source_release(filter);
 		};
 
 		auto redo = [scene_name, prev = std::string(prevName),
 			     name](const std::string &data) {
-			obs_source_t *ssource =
+			OBSSourceAutoRelease ssource =
 				obs_get_source_by_name(scene_name.c_str());
 			reinterpret_cast<OBSBasic *>(App()->GetMainWindow())
-				->SetCurrentScene(ssource, true);
-			obs_source_release(ssource);
+				->SetCurrentScene(ssource.Get(), true);
 
-			obs_source_t *source =
+			OBSSourceAutoRelease source =
 				obs_get_source_by_name(data.c_str());
-			obs_source_t *filter = obs_source_get_filter_by_name(
-				source, prev.c_str());
+			OBSSourceAutoRelease filter =
+				obs_source_get_filter_by_name(source,
+							      prev.c_str());
 			obs_source_set_name(filter, name.c_str());
-			obs_source_release(source);
-			obs_source_release(filter);
 		};
 
 		std::string undo_data(sourceName);
@@ -1133,9 +1148,12 @@ void OBSBasicFilters::ResetFilters()
 	if (!filter)
 		return;
 
-	obs_data_t *settings = obs_source_get_settings(filter);
+	OBSDataAutoRelease settings = obs_source_get_settings(filter);
+
+	OBSDataAutoRelease empty_settings = obs_data_create();
+	FilterChangeUndoRedo((void *)filter, settings, empty_settings);
+
 	obs_data_clear(settings);
-	obs_data_release(settings);
 
 	if (!view->DeferUpdate())
 		obs_source_update(filter, nullptr);
@@ -1161,9 +1179,9 @@ void OBSBasicFilters::PasteFilter()
 	if (!filter)
 		return;
 
-	obs_data_array_t *undo_array = obs_source_backup_filters(source);
+	OBSDataArrayAutoRelease undo_array = obs_source_backup_filters(source);
 	obs_source_copy_single_filter(source, filter);
-	obs_data_array_t *redo_array = obs_source_backup_filters(source);
+	OBSDataArrayAutoRelease redo_array = obs_source_backup_filters(source);
 
 	const char *filterName = obs_source_get_name(filter);
 	const char *sourceName = obs_source_get_name(source);
@@ -1172,14 +1190,11 @@ void OBSBasicFilters::PasteFilter()
 
 	main->CreateFilterPasteUndoRedoAction(text, source, undo_array,
 					      redo_array);
-
-	obs_data_array_release(undo_array);
-	obs_data_array_release(redo_array);
 }
 
 void OBSBasicFilters::delete_filter(OBSSource filter)
 {
-	obs_data_t *wrapper = obs_save_source(filter);
+	OBSDataAutoRelease wrapper = obs_save_source(filter);
 	std::string parent_name(obs_source_get_name(source));
 	obs_data_set_string(wrapper, "undo_name", parent_name.c_str());
 
@@ -1187,43 +1202,35 @@ void OBSBasicFilters::delete_filter(OBSSource filter)
 		reinterpret_cast<OBSBasic *>(App()->GetMainWindow())
 			->GetCurrentSceneSource());
 	auto undo = [scene_name](const std::string &data) {
-		obs_source_t *ssource =
+		OBSSourceAutoRelease ssource =
 			obs_get_source_by_name(scene_name.c_str());
 		reinterpret_cast<OBSBasic *>(App()->GetMainWindow())
-			->SetCurrentScene(ssource, true);
-		obs_source_release(ssource);
+			->SetCurrentScene(ssource.Get(), true);
 
-		obs_data_t *dat = obs_data_create_from_json(data.c_str());
-		obs_source_t *source = obs_get_source_by_name(
+		OBSDataAutoRelease dat =
+			obs_data_create_from_json(data.c_str());
+		OBSSourceAutoRelease source = obs_get_source_by_name(
 			obs_data_get_string(dat, "undo_name"));
-		obs_source_t *filter = obs_load_source(dat);
+		OBSSourceAutoRelease filter = obs_load_source(dat);
 		obs_source_filter_add(source, filter);
-
-		obs_data_release(dat);
-		obs_source_release(source);
-		obs_source_release(filter);
 	};
 
-	obs_data_t *rwrapper = obs_data_create();
+	OBSDataAutoRelease rwrapper = obs_data_create();
 	obs_data_set_string(rwrapper, "fname", obs_source_get_name(filter));
 	obs_data_set_string(rwrapper, "sname", parent_name.c_str());
 	auto redo = [scene_name](const std::string &data) {
-		obs_source_t *ssource =
+		OBSSourceAutoRelease ssource =
 			obs_get_source_by_name(scene_name.c_str());
 		reinterpret_cast<OBSBasic *>(App()->GetMainWindow())
-			->SetCurrentScene(ssource, true);
-		obs_source_release(ssource);
+			->SetCurrentScene(ssource.Get(), true);
 
-		obs_data_t *dat = obs_data_create_from_json(data.c_str());
-		obs_source_t *source = obs_get_source_by_name(
+		OBSDataAutoRelease dat =
+			obs_data_create_from_json(data.c_str());
+		OBSSourceAutoRelease source = obs_get_source_by_name(
 			obs_data_get_string(dat, "sname"));
-		obs_source_t *filter = obs_source_get_filter_by_name(
+		OBSSourceAutoRelease filter = obs_source_get_filter_by_name(
 			source, obs_data_get_string(dat, "fname"));
 		obs_source_filter_remove(source, filter);
-
-		obs_data_release(dat);
-		obs_source_release(filter);
-		obs_source_release(source);
 	};
 
 	std::string undo_data(obs_data_get_json(wrapper));
@@ -1232,7 +1239,4 @@ void OBSBasicFilters::delete_filter(OBSSource filter)
 		QTStr("Undo.Delete").arg(obs_source_get_name(filter)), undo,
 		redo, undo_data, redo_data, false);
 	obs_source_filter_remove(source, filter);
-
-	obs_data_release(wrapper);
-	obs_data_release(rwrapper);
 }
