@@ -106,7 +106,7 @@ private:
 
 	void ProcessClick(const vec2 &pos);
 
-	obs_data_t *wrapper = NULL;
+	OBSDataAutoRelease wrapper = nullptr;
 	bool changed;
 
 public:
