@@ -6,6 +6,7 @@
 #include "ui_OBSBasicTransform.h"
 
 class OBSBasic;
+class QListWidgetItem;
 
 class OBSBasicTransform : public QDialog {
 	Q_OBJECT
@@ -21,6 +22,8 @@ private:
 	OBSSignal selectSignal;
 	OBSSignal deselectSignal;
 
+	std::string undo_data;
+
 	bool ignoreTransformSignal = false;
 	bool ignoreItemChange = false;
 
@@ -46,4 +49,8 @@ private slots:
 
 public:
 	OBSBasicTransform(OBSBasic *parent);
+	~OBSBasicTransform();
+
+public slots:
+	void OnSceneChanged(QListWidgetItem *current, QListWidgetItem *prev);
 };
