@@ -21,7 +21,18 @@ static enum AVPixelFormat closest_format(enum AVPixelFormat fmt)
 	switch (fmt) {
 	case AV_PIX_FMT_YUYV422:
 		return AV_PIX_FMT_YUYV422;
+
 	case AV_PIX_FMT_YUV444P:
+	case AV_PIX_FMT_YUV444P16LE:
+	case AV_PIX_FMT_YUV444P16BE:
+	case AV_PIX_FMT_YUV444P9BE:
+	case AV_PIX_FMT_YUV444P9LE:
+	case AV_PIX_FMT_YUV444P10BE:
+	case AV_PIX_FMT_YUV444P10LE:
+	case AV_PIX_FMT_YUV444P12BE:
+	case AV_PIX_FMT_YUV444P12LE:
+	case AV_PIX_FMT_YUV444P14BE:
+	case AV_PIX_FMT_YUV444P14LE:
 		return AV_PIX_FMT_YUV444P;
 
 	case AV_PIX_FMT_YUV422P:
@@ -68,6 +79,16 @@ static enum AVPixelFormat closest_format(enum AVPixelFormat fmt)
 		return AV_PIX_FMT_YUVA422P;
 
 	case AV_PIX_FMT_YUVA444P:
+	case AV_PIX_FMT_YUVA444P9BE:
+	case AV_PIX_FMT_YUVA444P9LE:
+	case AV_PIX_FMT_YUVA444P10BE:
+	case AV_PIX_FMT_YUVA444P10LE:
+	case AV_PIX_FMT_YUVA444P16BE:
+	case AV_PIX_FMT_YUVA444P16LE:
+#if LIBAVUTIL_BUILD >= AV_VERSION_INT(56, 31, 100)
+	case AV_PIX_FMT_YUVA444P12BE:
+	case AV_PIX_FMT_YUVA444P12LE:
+#endif
 		return AV_PIX_FMT_YUVA444P;
 
 	case AV_PIX_FMT_RGBA:
