@@ -67,6 +67,8 @@ typedef HINSTANCE(WINAPI *SHELLEXECUTEA)(HWND hwnd, LPCTSTR operation,
 					 LPCTSTR file, LPCTSTR parameters,
 					 LPCTSTR directory, INT show_flags);
 
+typedef HRESULT(WINAPI *GETTHREADDESCRIPTION)(HANDLE thread, PWSTR *desc);
+
 struct stack_trace {
 	CONTEXT context;
 	DWORD64 instruction_ptr;
@@ -182,8 +184,10 @@ static inline void init_sym_info(struct exception_handler_data *data)
 		data->sym_refresh_module_list(data->process);
 
 	data->sym_info = LocalAlloc(LPTR, sizeof(*data->sym_info) + 256);
-	data->sym_info->SizeOfStruct = sizeof(SYMBOL_INFO);
-	data->sym_info->MaxNameLen = 256;
+	if (data->sym_info) {
+		data->sym_info->SizeOfStruct = sizeof(SYMBOL_INFO);
+		data->sym_info->MaxNameLen = 256;
+	}
 }
 
 static inline void init_version_info(struct exception_handler_data *data)
@@ -333,12 +337,16 @@ static inline bool walk_stack(struct exception_handler_data *data,
 		p = module_info.name_utf8;
 	}
 
-	success = !!data->sym_from_addr(data->process,
-					trace->frame.AddrPC.Offset,
-					&func_offset, data->sym_info);
+	if (data->sym_info) {
+		success = !!data->sym_from_addr(data->process,
+						trace->frame.AddrPC.Offset,
+						&func_offset, data->sym_info);
 
-	if (success)
-		os_wcs_to_utf8(data->sym_info->Name, 0, sym_name, 256);
+		if (success)
+			os_wcs_to_utf8(data->sym_info->Name, 0, sym_name, 256);
+	} else {
+		success = false;
+	}
 
 #ifdef _WIN64
 #define SUCCESS_FORMAT                         \
@@ -391,6 +399,40 @@ static inline bool walk_stack(struct exception_handler_data *data,
 	"Arg1     Arg2     Arg3     Address\r\n"
 #endif
 
+static inline char *get_thread_name(HANDLE thread)
+{
+	static GETTHREADDESCRIPTION get_thread_desc = NULL;
+	static bool failed = false;
+
+	if (!get_thread_desc) {
+		if (failed) {
+			return NULL;
+		}
+
+		HMODULE k32 = LoadLibraryW(L"kernel32.dll");
+		get_thread_desc = (GETTHREADDESCRIPTION)GetProcAddress(
+			k32, "GetThreadDescription");
+		if (!get_thread_desc) {
+			failed = true;
+			return NULL;
+		}
+	}
+
+	wchar_t *w_name;
+	HRESULT hr = get_thread_desc(thread, &w_name);
+	if (FAILED(hr) || !w_name) {
+		return NULL;
+	}
+
+	struct dstr name = {0};
+	dstr_from_wcs(&name, w_name);
+	if (name.len)
+		dstr_insert_ch(&name, 0, ' ');
+	LocalFree(w_name);
+
+	return name.array;
+}
+
 static inline void write_thread_trace(struct exception_handler_data *data,
 				      THREADENTRY32 *entry, bool first_thread)
 {
@@ -398,6 +440,7 @@ static inline void write_thread_trace(struct exception_handler_data *data,
 	struct stack_trace trace = {0};
 	struct stack_trace *ptrace;
 	HANDLE thread;
+	char *thread_name;
 
 	if (first_thread != crash_thread)
 		return;
@@ -413,8 +456,13 @@ static inline void write_thread_trace(struct exception_handler_data *data,
 	GetThreadContext(thread, &trace.context);
 	init_instruction_data(&trace);
 
-	dstr_catf(&data->str, "\r\nThread %lX%s\r\n" TRACE_TOP,
-		  entry->th32ThreadID, crash_thread ? " (Crashed)" : "");
+	thread_name = get_thread_name(thread);
+
+	dstr_catf(&data->str, "\r\nThread %lX:%s%s\r\n" TRACE_TOP,
+		  entry->th32ThreadID, thread_name ? thread_name : "",
+		  crash_thread ? " (Crashed)" : "");
+
+	bfree(thread_name);
 
 	ptrace = crash_thread ? &data->main_trace : &trace;
 
@@ -501,7 +549,7 @@ static LONG CALLBACK exception_handler(PEXCEPTION_POINTERS exception)
 	inside_handler = true;
 
 	handle_exception(&data, exception);
-	bcrash(data.str.array);
+	bcrash("%s", data.str.array);
 	exception_handler_data_free(&data);
 
 	inside_handler = false;
