@@ -23,21 +23,30 @@
 
 /* RAII wrappers */
 
+template<typename T, void release(T)> class OBSRefAutoRelease;
 template<typename T, void addref(T), void release(T)> class OBSRef;
+template<typename T, T getref(T), void release(T)> class OBSSafeRef;
 
-using OBSSource = OBSRef<obs_source_t *, obs_source_addref, obs_source_release>;
-using OBSScene = OBSRef<obs_scene_t *, obs_scene_addref, obs_scene_release>;
+using OBSObject =
+	OBSSafeRef<obs_object_t *, obs_object_get_ref, obs_object_release>;
+using OBSSource =
+	OBSSafeRef<obs_source_t *, obs_source_get_ref, obs_source_release>;
+using OBSScene =
+	OBSSafeRef<obs_scene_t *, obs_scene_get_ref, obs_scene_release>;
 using OBSSceneItem =
 	OBSRef<obs_sceneitem_t *, obs_sceneitem_addref, obs_sceneitem_release>;
 using OBSData = OBSRef<obs_data_t *, obs_data_addref, obs_data_release>;
 using OBSDataArray = OBSRef<obs_data_array_t *, obs_data_array_addref,
 			    obs_data_array_release>;
-using OBSOutput = OBSRef<obs_output_t *, obs_output_addref, obs_output_release>;
+using OBSOutput =
+	OBSSafeRef<obs_output_t *, obs_output_get_ref, obs_output_release>;
 using OBSEncoder =
-	OBSRef<obs_encoder_t *, obs_encoder_addref, obs_encoder_release>;
+	OBSSafeRef<obs_encoder_t *, obs_encoder_get_ref, obs_encoder_release>;
 using OBSService =
-	OBSRef<obs_service_t *, obs_service_addref, obs_service_release>;
+	OBSSafeRef<obs_service_t *, obs_service_get_ref, obs_service_release>;
 
+using OBSWeakObject = OBSRef<obs_weak_object_t *, obs_weak_object_addref,
+			     obs_weak_object_release>;
 using OBSWeakSource = OBSRef<obs_weak_source_t *, obs_weak_source_addref,
 			     obs_weak_source_release>;
 using OBSWeakOutput = OBSRef<obs_weak_output_t *, obs_weak_output_addref,
@@ -47,33 +56,57 @@ using OBSWeakEncoder = OBSRef<obs_weak_encoder_t *, obs_weak_encoder_addref,
 using OBSWeakService = OBSRef<obs_weak_service_t *, obs_weak_service_addref,
 			      obs_weak_service_release>;
 
-template<typename T, void addref(T), void release(T)> class OBSRef {
+#define OBS_AUTORELEASE
+using OBSObjectAutoRelease =
+	OBSRefAutoRelease<obs_object_t *, obs_object_release>;
+using OBSSourceAutoRelease =
+	OBSRefAutoRelease<obs_source_t *, obs_source_release>;
+using OBSSceneAutoRelease = OBSRefAutoRelease<obs_scene_t *, obs_scene_release>;
+using OBSSceneItemAutoRelease =
+	OBSRefAutoRelease<obs_sceneitem_t *, obs_sceneitem_release>;
+using OBSDataAutoRelease = OBSRefAutoRelease<obs_data_t *, obs_data_release>;
+using OBSDataArrayAutoRelease =
+	OBSRefAutoRelease<obs_data_array_t *, obs_data_array_release>;
+using OBSOutputAutoRelease =
+	OBSRefAutoRelease<obs_output_t *, obs_output_release>;
+using OBSEncoderAutoRelease =
+	OBSRefAutoRelease<obs_encoder_t *, obs_encoder_release>;
+using OBSServiceAutoRelease =
+	OBSRefAutoRelease<obs_service_t *, obs_service_release>;
+
+using OBSWeakObjectAutoRelease =
+	OBSRefAutoRelease<obs_weak_object_t *, obs_weak_object_release>;
+using OBSWeakSourceAutoRelease =
+	OBSRefAutoRelease<obs_weak_source_t *, obs_weak_source_release>;
+using OBSWeakOutputAutoRelease =
+	OBSRefAutoRelease<obs_weak_output_t *, obs_weak_output_release>;
+using OBSWeakEncoderAutoRelease =
+	OBSRefAutoRelease<obs_weak_encoder_t *, obs_weak_encoder_release>;
+using OBSWeakServiceAutoRelease =
+	OBSRefAutoRelease<obs_weak_service_t *, obs_weak_service_release>;
+
+template<typename T, void release(T)> class OBSRefAutoRelease {
+protected:
 	T val;
 
-	inline OBSRef &Replace(T valIn)
+public:
+	inline OBSRefAutoRelease() : val(nullptr) {}
+	inline OBSRefAutoRelease(T val_) : val(val_) {}
+	OBSRefAutoRelease(const OBSRefAutoRelease &ref) = delete;
+	inline OBSRefAutoRelease(OBSRefAutoRelease &&ref) : val(ref.val)
 	{
-		addref(valIn);
-		release(val);
-		val = valIn;
-		return *this;
+		ref.val = nullptr;
 	}
 
-	struct TakeOwnership {
-	};
-	inline OBSRef(T val, TakeOwnership) : val(val) {}
+	inline ~OBSRefAutoRelease() { release(val); }
 
-public:
-	inline OBSRef() : val(nullptr) {}
-	inline OBSRef(T val_) : val(val_) { addref(val); }
-	inline OBSRef(const OBSRef &ref) : val(ref.val) { addref(val); }
-	inline OBSRef(OBSRef &&ref) : val(ref.val) { ref.val = nullptr; }
+	inline operator T() const { return val; }
+	inline T Get() const { return val; }
 
-	inline ~OBSRef() { release(val); }
+	inline bool operator==(T p) const { return val == p; }
+	inline bool operator!=(T p) const { return val != p; }
 
-	inline OBSRef &operator=(T valIn) { return Replace(valIn); }
-	inline OBSRef &operator=(const OBSRef &ref) { return Replace(ref.val); }
-
-	inline OBSRef &operator=(OBSRef &&ref)
+	inline OBSRefAutoRelease &operator=(OBSRefAutoRelease &&ref)
 	{
 		if (this != &ref) {
 			release(val);
@@ -84,25 +117,116 @@ public:
 		return *this;
 	}
 
-	inline operator T() const { return val; }
-	inline T Get() const { return val; }
+	inline OBSRefAutoRelease &operator=(T new_val)
+	{
+		release(val);
+		val = new_val;
+		return *this;
+	}
+};
 
-	inline bool operator==(T p) const { return val == p; }
-	inline bool operator!=(T p) const { return val != p; }
+template<typename T, void addref(T), void release(T)>
+class OBSRef : public OBSRefAutoRelease<T, release> {
 
-	friend OBSSource OBSGetStrongRef(obs_weak_source_t *weak);
+	inline OBSRef &Replace(T valIn)
+	{
+		addref(valIn);
+		release(this->val);
+		this->val = valIn;
+		return *this;
+	}
+
+	struct TakeOwnership {
+	};
+	inline OBSRef(T val_, TakeOwnership)
+		: OBSRefAutoRelease<T, release>::OBSRefAutoRelease(val_)
+	{
+	}
+
+public:
+	inline OBSRef()
+		: OBSRefAutoRelease<T, release>::OBSRefAutoRelease(nullptr)
+	{
+	}
+	inline OBSRef(const OBSRef &ref)
+		: OBSRefAutoRelease<T, release>::OBSRefAutoRelease(ref.val)
+	{
+		addref(this->val);
+	}
+	inline OBSRef(T val_)
+		: OBSRefAutoRelease<T, release>::OBSRefAutoRelease(val_)
+	{
+		addref(this->val);
+	}
+
+	inline OBSRef &operator=(const OBSRef &ref) { return Replace(ref.val); }
+	inline OBSRef &operator=(T valIn) { return Replace(valIn); }
+
+	friend OBSWeakObject OBSGetWeakRef(obs_object_t *object);
 	friend OBSWeakSource OBSGetWeakRef(obs_source_t *source);
-
-	friend OBSOutput OBSGetStrongRef(obs_weak_output_t *weak);
 	friend OBSWeakOutput OBSGetWeakRef(obs_output_t *output);
-
-	friend OBSEncoder OBSGetStrongRef(obs_weak_encoder_t *weak);
 	friend OBSWeakEncoder OBSGetWeakRef(obs_encoder_t *encoder);
-
-	friend OBSService OBSGetStrongRef(obs_weak_service_t *weak);
 	friend OBSWeakService OBSGetWeakRef(obs_service_t *service);
 };
 
+template<typename T, T getref(T), void release(T)>
+class OBSSafeRef : public OBSRefAutoRelease<T, release> {
+
+	inline OBSSafeRef &Replace(T valIn)
+	{
+		T newVal = getref(valIn);
+		release(this->val);
+		this->val = newVal;
+		return *this;
+	}
+
+	struct TakeOwnership {
+	};
+	inline OBSSafeRef(T val_, TakeOwnership)
+		: OBSRefAutoRelease<T, release>::OBSRefAutoRelease(val_)
+	{
+	}
+
+public:
+	inline OBSSafeRef()
+		: OBSRefAutoRelease<T, release>::OBSRefAutoRelease(nullptr)
+	{
+	}
+	inline OBSSafeRef(const OBSSafeRef &ref)
+		: OBSRefAutoRelease<T, release>::OBSRefAutoRelease(ref.val)
+	{
+		this->val = getref(ref.val);
+	}
+	inline OBSSafeRef(T val_)
+		: OBSRefAutoRelease<T, release>::OBSRefAutoRelease(val_)
+	{
+		this->val = getref(this->val);
+	}
+
+	inline OBSSafeRef &operator=(const OBSSafeRef &ref)
+	{
+		return Replace(ref.val);
+	}
+	inline OBSSafeRef &operator=(T valIn) { return Replace(valIn); }
+
+	friend OBSObject OBSGetStrongRef(obs_weak_object_t *weak);
+	friend OBSSource OBSGetStrongRef(obs_weak_source_t *weak);
+	friend OBSOutput OBSGetStrongRef(obs_weak_output_t *weak);
+	friend OBSEncoder OBSGetStrongRef(obs_weak_encoder_t *weak);
+	friend OBSService OBSGetStrongRef(obs_weak_service_t *weak);
+};
+
+inline OBSObject OBSGetStrongRef(obs_weak_object_t *weak)
+{
+	return {obs_weak_object_get_object(weak), OBSObject::TakeOwnership()};
+}
+
+inline OBSWeakObject OBSGetWeakRef(obs_object_t *object)
+{
+	return {obs_object_get_weak_object(object),
+		OBSWeakObject::TakeOwnership()};
+}
+
 inline OBSSource OBSGetStrongRef(obs_weak_source_t *weak)
 {
 	return {obs_weak_source_get_source(weak), OBSSource::TakeOwnership()};
@@ -150,26 +274,26 @@ inline OBSWeakService OBSGetWeakRef(obs_service_t *service)
 }
 
 /* objects that are not meant to be instanced */
-template<typename T, void destroy(T)> class OBSObj {
+template<typename T, void destroy(T)> class OBSPtr {
 	T obj;
 
 public:
-	inline OBSObj() : obj(nullptr) {}
-	inline OBSObj(T obj_) : obj(obj_) {}
-	inline OBSObj(const OBSObj &) = delete;
-	inline OBSObj(OBSObj &&other) : obj(other.obj) { other.obj = nullptr; }
+	inline OBSPtr() : obj(nullptr) {}
+	inline OBSPtr(T obj_) : obj(obj_) {}
+	inline OBSPtr(const OBSPtr &) = delete;
+	inline OBSPtr(OBSPtr &&other) : obj(other.obj) { other.obj = nullptr; }
 
-	inline ~OBSObj() { destroy(obj); }
+	inline ~OBSPtr() { destroy(obj); }
 
-	inline OBSObj &operator=(T obj_)
+	inline OBSPtr &operator=(T obj_)
 	{
 		if (obj_ != obj)
 			destroy(obj);
 		obj = obj_;
 		return *this;
 	}
-	inline OBSObj &operator=(const OBSObj &) = delete;
-	inline OBSObj &operator=(OBSObj &&other)
+	inline OBSPtr &operator=(const OBSPtr &) = delete;
+	inline OBSPtr &operator=(OBSPtr &&other)
 	{
 		if (obj)
 			destroy(obj);
@@ -184,8 +308,8 @@ public:
 	inline bool operator!=(T p) const { return obj != p; }
 };
 
-using OBSDisplay = OBSObj<obs_display_t *, obs_display_destroy>;
-using OBSView = OBSObj<obs_view_t *, obs_view_destroy>;
+using OBSDisplay = OBSPtr<obs_display_t *, obs_display_destroy>;
+using OBSView = OBSPtr<obs_view_t *, obs_view_destroy>;
 
 /* signal handler connection */
 class OBSSignal {
