@@ -25,12 +25,20 @@
 
 #ifdef _MSC_VER
 #define OBS_DEPRECATED __declspec(deprecated)
+#define OBS_NORETURN __declspec(noreturn)
 #define FORCE_INLINE __forceinline
 #else
 #define OBS_DEPRECATED __attribute__((deprecated))
+#define OBS_NORETURN __attribute__((noreturn))
 #define FORCE_INLINE inline __attribute__((always_inline))
 #endif
 
+#if defined(IS_LIBOBS) || defined(SWIG)
+#define OBS_EXTERNAL_DEPRECATED
+#else
+#define OBS_EXTERNAL_DEPRECATED OBS_DEPRECATED
+#endif
+
 #ifdef _MSC_VER
 #define EXPORT __declspec(dllexport)
 #else
