@@ -66,6 +66,20 @@ const std::string &DeckLinkDeviceMode::GetName(void) const
 	return name;
 }
 
+bool DeckLinkDeviceMode::IsEqualFrameRate(int64_t num, int64_t den)
+{
+	bool equal = false;
+
+	if (mode) {
+		BMDTimeValue frameDuration;
+		BMDTimeScale timeScale;
+		if (SUCCEEDED(mode->GetFrameRate(&frameDuration, &timeScale)))
+			equal = timeScale * den == frameDuration * num;
+	}
+
+	return equal;
+}
+
 void DeckLinkDeviceMode::SetMode(IDeckLinkDisplayMode *mode_)
 {
 	IDeckLinkDisplayMode *old = mode;
