@@ -22,9 +22,15 @@
 
 #include "Defines.h"
 
+#ifdef DEBUG
 #define DLog(fmt, ...) NSLog((PLUGIN_NAME @"(DAL): " fmt), ##__VA_ARGS__)
 #define DLogFunc(fmt, ...) \
 	NSLog((PLUGIN_NAME @"(DAL): %s " fmt), __FUNCTION__, ##__VA_ARGS__)
+#else
+#define DLog(fmt, ...) (void)(fmt, ##__VA_ARGS__)
+#define DLogFunc(fmt, ...) (void)(fmt, __FUNCTION__, ##__VA_ARGS__)
+#endif
+
 #define VLog(fmt, ...)
 #define VLogFunc(fmt, ...)
 #define ELog(fmt, ...) DLog(fmt, ##__VA_ARGS__)
