@@ -19,22 +19,6 @@
 #define VT_BLOG(level, format, ...) \
 	VT_LOG_ENCODER(enc->encoder, level, format, ##__VA_ARGS__)
 
-// Clipped from NSApplication as it is in a ObjC header
-extern const double NSAppKitVersionNumber;
-#define NSAppKitVersionNumber10_8 1187
-
-#define APPLE_H264_ENC_ID_HW "com.apple.videotoolbox.videoencoder.h264.gva"
-#define APPLE_H264_ENC_ID_SW "com.apple.videotoolbox.videoencoder.h264"
-
-// Get around missing symbol on 10.8 during compilation
-enum { kCMFormatDescriptionBridgeError_InvalidParameter_ = -12712,
-};
-
-static bool is_appkit10_9_or_greater()
-{
-	return floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_8;
-}
-
 static DARRAY(struct vt_encoder {
 	const char *name;
 	const char *disp_name;
@@ -124,6 +108,16 @@ static CFStringRef obs_to_vt_colorspace(enum video_colorspace cs)
 	if ((code = (x)) != noErr) \
 		return code;
 
+static OSStatus session_set_prop_float(VTCompressionSessionRef session,
+				       CFStringRef key, float val)
+{
+	CFNumberRef n = CFNumberCreate(NULL, kCFNumberFloat32Type, &val);
+	OSStatus code = VTSessionSetProperty(session, key, n);
+	CFRelease(n);
+
+	return code;
+}
+
 static OSStatus session_set_prop_int(VTCompressionSessionRef session,
 				     CFStringRef key, int32_t val)
 {
@@ -316,22 +310,22 @@ static bool create_encoder(struct vt_h264_encoder *enc)
 	STATUS_CHECK(session_set_prop_int(
 		s, kVTCompressionPropertyKey_MaxKeyFrameInterval,
 		enc->keyint * ((float)enc->fps_num / enc->fps_den)));
-	STATUS_CHECK(session_set_prop_int(
+	STATUS_CHECK(session_set_prop_float(
 		s, kVTCompressionPropertyKey_ExpectedFrameRate,
-		ceil((float)enc->fps_num / enc->fps_den)));
+		(float)enc->fps_num / enc->fps_den));
 	STATUS_CHECK(session_set_prop(
 		s, kVTCompressionPropertyKey_AllowFrameReordering,
 		enc->bframes ? kCFBooleanTrue : kCFBooleanFalse));
 
 	// This can fail depending on hardware configuration
 	code = session_set_prop(s, kVTCompressionPropertyKey_RealTime,
-				kCFBooleanTrue);
+				kCFBooleanFalse);
 	if (code != noErr)
-		log_osstatus(LOG_WARNING, enc,
-			     "setting "
-			     "kVTCompressionPropertyKey_RealTime, "
-			     "frame delay might be increased",
-			     code);
+		log_osstatus(
+			LOG_WARNING, enc,
+			"setting kVTCompressionPropertyKey_RealTime failed, "
+			"frame delay might be increased",
+			code);
 
 	STATUS_CHECK(session_set_prop(s, kVTCompressionPropertyKey_ProfileLevel,
 				      obs_to_vt_profile(enc->profile)));
@@ -490,15 +484,14 @@ static bool vt_h264_update(void *data, obs_data_t *settings)
 	return true;
 }
 
-static void *vt_h264_create(obs_data_t *settings, obs_encoder_t *encoder,
-			    const char *vt_encoder_id)
+static void *vt_h264_create(obs_data_t *settings, obs_encoder_t *encoder)
 {
 	struct vt_h264_encoder *enc = bzalloc(sizeof(struct vt_h264_encoder));
 
 	OSStatus code;
 
 	enc->encoder = encoder;
-	enc->vt_encoder_id = vt_encoder_id;
+	enc->vt_encoder_id = obs_encoder_get_id(encoder);
 
 	update_params(enc, settings);
 
@@ -516,16 +509,6 @@ fail:
 	return NULL;
 }
 
-static void *vt_h264_create_hw(obs_data_t *settings, obs_encoder_t *encoder)
-{
-	return vt_h264_create(settings, encoder, APPLE_H264_ENC_ID_HW);
-}
-
-static void *vt_h264_create_sw(obs_data_t *settings, obs_encoder_t *encoder)
-{
-	return vt_h264_create(settings, encoder, APPLE_H264_ENC_ID_SW);
-}
-
 static const uint8_t annexb_startcode[4] = {0, 0, 0, 1};
 
 static void packet_put(struct darray *packet, const uint8_t *buf, size_t size)
@@ -629,7 +612,7 @@ static bool convert_sample_to_annexb(struct vt_h264_encoder *enc,
 		format_desc, 0, NULL, NULL, &param_count, &nal_length_bytes);
 	// it is not clear what errors this function can return
 	// so we check the two most reasonable
-	if (code == kCMFormatDescriptionBridgeError_InvalidParameter_ ||
+	if (code == kCMFormatDescriptionBridgeError_InvalidParameter ||
 	    code == kCMFormatDescriptionError_InvalidParameter) {
 		VT_BLOG(LOG_WARNING, "assuming 2 parameter sets "
 				     "and 4 byte NAL length header");
@@ -682,8 +665,10 @@ static bool parse_sample(struct vt_h264_encoder *enc, CMSampleBufferRef buffer,
 	dts = CMTimeMultiplyByFloat64(dts,
 				      ((Float64)enc->fps_num / enc->fps_den));
 
+	if (CMTIME_IS_INVALID(dts))
+		dts = pts;
 	// imitate x264's negative dts when bframes might have pts < dts
-	if (enc->bframes)
+	else if (enc->bframes)
 		dts = CMTimeSubtract(dts, off);
 
 	bool keyframe = is_sample_keyframe(buffer);
@@ -845,16 +830,16 @@ static bool vt_h264_extra_data(void *data, uint8_t **extra_data, size_t *size)
 	return true;
 }
 
-static const char *vt_h264_getname_hw(void *unused)
+static const char *vt_h264_getname(void *data)
 {
-	UNUSED_PARAMETER(unused);
-	return obs_module_text("VTH264EncHW");
-}
+	const char *disp_name = vt_encoders.array[(int)data].disp_name;
 
-static const char *vt_h264_getname_sw(void *unused)
-{
-	UNUSED_PARAMETER(unused);
-	return obs_module_text("VTH264EncSW");
+	if (strcmp("Apple H.264 (HW)", disp_name) == 0) {
+		return obs_module_text("VTH264EncHW");
+	} else if (strcmp("Apple H.264 (SW)", disp_name) == 0) {
+		return obs_module_text("VTH264EncSW");
+	}
+	return disp_name;
 }
 
 #define TEXT_VT_ENCODER obs_module_text("VTEncoder")
@@ -994,30 +979,16 @@ void register_encoders()
 	};
 
 	for (size_t i = 0; i < vt_encoders.num; i++) {
-		if (strcmp(vt_encoders.array[i].id, APPLE_H264_ENC_ID_HW) ==
-		    0) {
-			info.id = "vt_h264_hw";
-			info.get_name = vt_h264_getname_hw;
-			info.create = vt_h264_create_hw;
-			obs_register_encoder(&info);
-		} else if (strcmp(vt_encoders.array[i].id,
-				  APPLE_H264_ENC_ID_SW) == 0) {
-			info.id = "vt_h264_sw";
-			info.get_name = vt_h264_getname_sw;
-			info.create = vt_h264_create_sw;
-			obs_register_encoder(&info);
-		}
+		info.id = vt_encoders.array[i].id;
+		info.type_data = (void *)i;
+		info.get_name = vt_h264_getname;
+		info.create = vt_h264_create;
+		obs_register_encoder(&info);
 	}
 }
 
 bool obs_module_load(void)
 {
-	if (!is_appkit10_9_or_greater()) {
-		VT_LOG(LOG_WARNING, "Not adding VideoToolbox H264 encoder; "
-				    "AppKit must be version 10.9 or greater");
-		return false;
-	}
-
 	encoder_list_create();
 	register_encoders();
 
