@@ -66,6 +66,8 @@ static void ffmpeg_mux_destroy(void *data)
 	replay_buffer_clear(stream);
 	if (stream->mux_thread_joinable)
 		pthread_join(stream->mux_thread, NULL);
+	for (size_t i = 0; i < stream->mux_packets.num; i++)
+		obs_encoder_packet_release(&stream->mux_packets.array[i]);
 	da_free(stream->mux_packets);
 	circlebuf_free(&stream->packets);
 
@@ -167,8 +169,9 @@ static void add_audio_encoder_params(struct dstr *cmd, obs_encoder_t *aencoder)
 	dstr_copy(&name, obs_encoder_get_name(aencoder));
 	dstr_replace(&name, "\"", "\"\"");
 
-	dstr_catf(cmd, "\"%s\" %d %d %d ", name.array, bitrate,
+	dstr_catf(cmd, "\"%s\" %d %d %d %d ", name.array, bitrate,
 		  (int)obs_encoder_get_sample_rate(aencoder),
+		  (int)obs_encoder_get_frame_size(aencoder),
 		  (int)audio_output_get_channels(audio));
 
 	dstr_free(&name);
@@ -609,7 +612,7 @@ struct obs_output_info ffmpeg_mpegts_muxer = {
 	.id = "ffmpeg_mpegts_muxer",
 	.flags = OBS_OUTPUT_AV | OBS_OUTPUT_ENCODED | OBS_OUTPUT_MULTI_TRACK |
 		 OBS_OUTPUT_SERVICE,
-	.encoded_video_codecs = "h264",
+	.encoded_video_codecs = "h264;av1",
 	.encoded_audio_codecs = "aac",
 	.get_name = ffmpeg_mpegts_mux_getname,
 	.create = ffmpeg_mux_create,
@@ -650,12 +653,6 @@ static void replay_buffer_hotkey(void *data, obs_hotkey_id id,
 			return;
 		}
 
-		calldata_t cd = {0};
-
-		signal_handler_t *sh =
-			obs_output_get_signal_handler(stream->output);
-		signal_handler_signal(sh, "saved", &cd);
-
 		stream->save_ts = os_gettime_ns() / 1000LL;
 	}
 }
@@ -730,6 +727,9 @@ static bool purge_front(struct ffmpeg_muxer *stream)
 	struct encoder_packet pkt;
 	bool keyframe;
 
+	if (!stream->packets.size)
+		return false;
+
 	circlebuf_pop_front(&stream->packets, &pkt, sizeof(pkt));
 
 	keyframe = pkt.type == OBS_ENCODER_VIDEO && pkt.keyframe;
@@ -757,6 +757,8 @@ static inline void purge(struct ffmpeg_muxer *stream)
 		struct encoder_packet pkt;
 
 		for (;;) {
+			if (!stream->packets.size)
+				return;
 			circlebuf_peek_front(&stream->packets, &pkt,
 					     sizeof(pkt));
 			if (pkt.type == OBS_ENCODER_VIDEO && pkt.keyframe)
@@ -788,7 +790,7 @@ static inline void replay_buffer_purge(struct ffmpeg_muxer *stream,
 
 static void insert_packet(struct darray *array, struct encoder_packet *packet,
 			  int64_t video_offset, int64_t *audio_offsets,
-			  int64_t video_dts_offset, int64_t *audio_dts_offsets)
+			  int64_t video_pts_offset, int64_t *audio_dts_offsets)
 {
 	struct encoder_packet pkt;
 	DARRAY(struct encoder_packet) packets;
@@ -799,8 +801,8 @@ static void insert_packet(struct darray *array, struct encoder_packet *packet,
 
 	if (pkt.type == OBS_ENCODER_VIDEO) {
 		pkt.dts_usec -= video_offset;
-		pkt.dts -= video_dts_offset;
-		pkt.pts -= video_dts_offset;
+		pkt.dts -= video_pts_offset;
+		pkt.pts -= video_pts_offset;
 	} else {
 		pkt.dts_usec -= audio_offsets[pkt.track_idx];
 		pkt.dts -= audio_dts_offsets[pkt.track_idx];
@@ -820,17 +822,20 @@ static void insert_packet(struct darray *array, struct encoder_packet *packet,
 static void *replay_buffer_mux_thread(void *data)
 {
 	struct ffmpeg_muxer *stream = data;
+	bool error = false;
 
 	start_pipe(stream, stream->path.array);
 
 	if (!stream->pipe) {
 		warn("Failed to create process pipe");
+		error = true;
 		goto error;
 	}
 
 	if (!send_headers(stream)) {
 		warn("Could not write headers for file '%s'",
 		     stream->path.array);
+		error = true;
 		goto error;
 	}
 
@@ -845,8 +850,21 @@ static void *replay_buffer_mux_thread(void *data)
 error:
 	os_process_pipe_destroy(stream->pipe);
 	stream->pipe = NULL;
+	if (error) {
+		for (size_t i = 0; i < stream->mux_packets.num; i++)
+			obs_encoder_packet_release(
+				&stream->mux_packets.array[i]);
+	}
 	da_free(stream->mux_packets);
 	os_atomic_set_bool(&stream->muxing, false);
+
+	if (!error) {
+		calldata_t cd = {0};
+		signal_handler_t *sh =
+			obs_output_get_signal_handler(stream->output);
+		signal_handler_signal(sh, "saved", &cd);
+	}
+
 	return NULL;
 }
 
@@ -863,7 +881,7 @@ static void replay_buffer_save(struct ffmpeg_muxer *stream)
 	bool found_video = false;
 	bool found_audio[MAX_AUDIO_MIXES] = {0};
 	int64_t video_offset = 0;
-	int64_t video_dts_offset = 0;
+	int64_t video_pts_offset = 0;
 	int64_t audio_offsets[MAX_AUDIO_MIXES] = {0};
 	int64_t audio_dts_offsets[MAX_AUDIO_MIXES] = {0};
 
@@ -873,8 +891,9 @@ static void replay_buffer_save(struct ffmpeg_muxer *stream)
 
 		if (pkt->type == OBS_ENCODER_VIDEO) {
 			if (!found_video) {
-				video_offset = pkt->dts_usec;
-				video_dts_offset = pkt->dts;
+				video_pts_offset = pkt->pts;
+				video_offset = video_pts_offset * 1000000 /
+					       pkt->timebase_den;
 				found_video = true;
 			}
 		} else {
@@ -886,7 +905,7 @@ static void replay_buffer_save(struct ffmpeg_muxer *stream)
 		}
 
 		insert_packet(&stream->mux_packets.da, pkt, video_offset,
-			      audio_offsets, video_dts_offset,
+			      audio_offsets, video_pts_offset,
 			      audio_dts_offsets);
 	}
 
@@ -923,6 +942,10 @@ static void replay_buffer_save(struct ffmpeg_muxer *stream)
 	stream->mux_thread_joinable = pthread_create(&stream->mux_thread, NULL,
 						     replay_buffer_mux_thread,
 						     stream) == 0;
+	if (!stream->mux_thread_joinable) {
+		warn("Failed to create muxer thread");
+		os_atomic_set_bool(&stream->muxing, false);
+	}
 }
 
 static void deactivate_replay_buffer(struct ffmpeg_muxer *stream, int code)
