@@ -9,20 +9,27 @@ MODULE_EXPORT const char *obs_module_description(void)
 }
 
 extern struct obs_source_info mask_filter;
+extern struct obs_source_info mask_filter_v2;
 extern struct obs_source_info crop_filter;
 extern struct obs_source_info gain_filter;
 extern struct obs_source_info color_filter;
+extern struct obs_source_info color_filter_v2;
 extern struct obs_source_info scale_filter;
 extern struct obs_source_info scroll_filter;
 extern struct obs_source_info gpu_delay_filter;
 extern struct obs_source_info color_key_filter;
+extern struct obs_source_info color_key_filter_v2;
 extern struct obs_source_info color_grade_filter;
 extern struct obs_source_info sharpness_filter;
+extern struct obs_source_info sharpness_filter_v2;
 extern struct obs_source_info chroma_key_filter;
+extern struct obs_source_info chroma_key_filter_v2;
 extern struct obs_source_info async_delay_filter;
 #if NOISEREDUCTION_ENABLED
 extern struct obs_source_info noise_suppress_filter;
 extern struct obs_source_info noise_suppress_filter_v2;
+extern bool load_nvafx(void);
+extern void unload_nvafx(void);
 #endif
 extern struct obs_source_info invert_polarity_filter;
 extern struct obs_source_info noise_gate_filter;
@@ -30,22 +37,32 @@ extern struct obs_source_info compressor_filter;
 extern struct obs_source_info limiter_filter;
 extern struct obs_source_info expander_filter;
 extern struct obs_source_info luma_key_filter;
+extern struct obs_source_info luma_key_filter_v2;
 
 bool obs_module_load(void)
 {
 	obs_register_source(&mask_filter);
+	obs_register_source(&mask_filter_v2);
 	obs_register_source(&crop_filter);
 	obs_register_source(&gain_filter);
 	obs_register_source(&color_filter);
+	obs_register_source(&color_filter_v2);
 	obs_register_source(&scale_filter);
 	obs_register_source(&scroll_filter);
 	obs_register_source(&gpu_delay_filter);
 	obs_register_source(&color_key_filter);
+	obs_register_source(&color_key_filter_v2);
 	obs_register_source(&color_grade_filter);
 	obs_register_source(&sharpness_filter);
+	obs_register_source(&sharpness_filter_v2);
 	obs_register_source(&chroma_key_filter);
+	obs_register_source(&chroma_key_filter_v2);
 	obs_register_source(&async_delay_filter);
 #if NOISEREDUCTION_ENABLED
+#ifdef LIBNVAFX_ENABLED
+	/* load nvidia audio fx dll */
+	load_nvafx();
+#endif
 	obs_register_source(&noise_suppress_filter);
 	obs_register_source(&noise_suppress_filter_v2);
 #endif
@@ -55,5 +72,13 @@ bool obs_module_load(void)
 	obs_register_source(&limiter_filter);
 	obs_register_source(&expander_filter);
 	obs_register_source(&luma_key_filter);
+	obs_register_source(&luma_key_filter_v2);
 	return true;
 }
+
+#ifdef LIBNVAFX_ENABLED
+void obs_module_unload(void)
+{
+	unload_nvafx();
+}
+#endif
