@@ -7,7 +7,9 @@
 
 #include "rtmp-format-ver.h"
 #include "lookup-config.h"
-#include "showroom.h"
+
+#include "service-specific/showroom.h"
+#include "service-specific/dacast.h"
 
 OBS_DECLARE_MODULE()
 OBS_MODULE_USE_DEFAULT_LOCALE("rtmp-services", "en-US")
@@ -72,6 +74,7 @@ static void refresh_callback(void *unused, calldata_t *cd)
 bool obs_module_load(void)
 {
 	init_twitch_data();
+	init_dacast_data();
 
 	dstr_copy(&module_name, "rtmp-services plugin (libobs ");
 	dstr_cat(&module_name, obs_get_version_string());
@@ -84,12 +87,14 @@ bool obs_module_load(void)
 #if !defined(_WIN32) || CHECK_FOR_SERVICE_UPDATES
 	char *local_dir = obs_module_file("");
 	char *cache_dir = obs_module_config_path("");
+	char update_url[128];
+	snprintf(update_url, sizeof(update_url), "%s/v%d", RTMP_SERVICES_URL,
+		 RTMP_SERVICES_FORMAT_VERSION);
 
 	if (cache_dir) {
 		update_info = update_info_create(RTMP_SERVICES_LOG_STR,
-						 module_name.array,
-						 RTMP_SERVICES_URL, local_dir,
-						 cache_dir,
+						 module_name.array, update_url,
+						 local_dir, cache_dir,
 						 confirm_service_file, NULL);
 	}
 
@@ -109,5 +114,6 @@ void obs_module_unload(void)
 	update_info_destroy(update_info);
 	unload_twitch_data();
 	free_showroom_data();
+	unload_dacast_data();
 	dstr_free(&module_name);
 }
