@@ -71,8 +71,7 @@ struct ft2_source {
 
 extern FT_Library ft2_lib;
 
-static void *ft2_source_create_v1(obs_data_t *settings, obs_source_t *source);
-static void *ft2_source_create_v2(obs_data_t *settings, obs_source_t *source);
+static void *ft2_source_create(obs_data_t *settings, obs_source_t *source);
 static void ft2_source_destroy(void *data);
 static void ft2_source_update(void *data, obs_data_t *settings);
 static void ft2_source_render(void *data, gs_effect_t *effect);
@@ -84,6 +83,9 @@ void draw_drop_shadow(struct ft2_source *srcdata);
 static uint32_t ft2_source_get_width(void *data);
 static uint32_t ft2_source_get_height(void *data);
 
+static void ft2_source_defaults_v1(obs_data_t *settings);
+static void ft2_source_defaults_v2(obs_data_t *settings);
+
 static obs_properties_t *ft2_source_properties(void *unused);
 
 static const char *ft2_source_get_name(void *unused);
